"""Get started by importing the Client.

```python
>>> from acrclient import Client

```

Create an instance and pass it a bearer token:

```python
>>> client = Client("bearer-token")

```

See [`Client`](./client/#acrclient.client.Client) for available methods and options.

"""

from .client import Client  # noqa: F401
